##Library Header
# $Id: $
# Copyright  2003-2005 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_rip_api.tcl
#
# Purpose:
#     A script development library containing RIP APIs for test automation with 
#     the Ixia chassis.
#
# Author:
#    Lavinia Raicea
#
# Usage:
#    package req Ixia
#
# Description:
#    The procedures contained within this library include:
#    - emulation_rip_config
#    - emulation_rip_control
#    - emulation_rip_route_config
#
# Requirements:
#     ixiaapiutils.tcl , a library containing TCL utilities
#     parseddashedargs.tcl , a library containing the proceDescr and 
#     parsedashedargds.tcl
#
# Variables:
#    To be added
#
# Keywords:
#    To be define
#
# Category:
#    To be define
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################


proc ::ixia::emulation_rip_config { args } {
    variable executeOnTclServer
	
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rip_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable rip_router_handles_array
    variable rip_route_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode           CHOICES create delete modify enable disable
    }
    set opt_args {
        -port_handle                 REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -handle
        -session_type                CHOICES ripv1 ripv2 ripng
        -count                       NUMERIC
                                     DEFAULT 1
        -gre_checksum_enable         CHOICES 0 1
                                     DEFAULT 0
        -gre_count                   NUMERIC
                                     DEFAULT 0
        -gre_dst_ip_addr             IP
        -gre_dst_ip_addr_step        IP
        -gre_dst_ip_addr_cstep       IP
        -gre_ipv6_addr               IPV6
        -gre_ipv6_addr_step          IPV6
                                     DEFAULT 0:0:0:1::0
        -gre_ipv6_addr_cstep         IPV6
        -gre_ipv6_prefix_length      RANGE   1-128
                                     DEFAULT 64
        -gre_key_enable              CHOICES 0 1
                                     DEFAULT 0
        -gre_key_in                  RANGE 0-4294967295
                                     DEFAULT 0
        -gre_key_in_step             RANGE 0-4294967295
                                     DEFAULT 0
        -gre_key_out                 RANGE 0-4294967295
                                     DEFAULT 0
        -gre_key_out_step            RANGE 0-4294967295
                                     DEFAULT 0
        -gre_seq_enable              CHOICES 0 1
                                     DEFAULT 0
        -gre_src_ip_addr_mode        CHOICES routed connected
                                     DEFAULT connected
        -intf_count                  NUMERIC
                                     DEFAULT 1
        -intf_ip_addr                IP
        -intf_prefix_length          RANGE 1-128
        -intf_ip_addr_step           IP
        -interface_metric            REGEXP 0\+|0|1|2|3|4|5|6|7|8|9|10|11|12|13|14|15|16
        -interface_handle
        -neighbor_intf_ip_addr       IP
        -neighbor_intf_ip_addr_step  IP
        -update_interval             RANGE 1-1000
        -update_interval_offset      RANGE 0-15
        -authentication_mode         CHOICES null text md5
        -password                    ALPHANUM
        -update_mode                 CHOICES no_horizon split_horizon
                                     CHOICES poison_reverse discard
        -vpi                         RANGE 0-255
        -vci                         RANGE 0-65535
        -vpi_step                    RANGE 0-255
        -vci_step                    RANGE 0-65535
        -atm_encapsulation           CHOICES VccMuxIPV4Routed
                                     CHOICES VccMuxIPV6Routed
                                     CHOICES VccMuxBridgedEthernetFCS
                                     CHOICES VccMuxBridgedEthernetNoFCS
                                     CHOICES LLCRoutedCLIP
                                     CHOICES LLCBridgedEthernetFCS
                                     CHOICES LLCBridgedEthernetNoFCS
        -vlan                        CHOICES 0 1
        -vlan_id                     RANGE   0-4095
        -vlan_id_mode                CHOICES fixed increment
                                     DEFAULT increment
        -vlan_id_step                RANGE   0-4096
                                     DEFAULT 1
        -vlan_user_priority          RANGE   0-7
                                     DEFAULT 0
        -router_id                   RANGE 0-65535
        -router_id_step              RANGE 0-65535
                                     DEFAULT 1
        -send_type                   CHOICES multicast broadcast_v1
                                     CHOICES broadcast_v2
        -receive_type                CHOICES v1 v2 v1_v2 ignore store
        -time_period                 RANGE 0-999999
        -num_routes_per_period       RANGE 0-999999
        -reset                       FLAG
        -mac_address_init            MAC
        -mac_address_step            MAC
                                     DEFAULT 0000.0000.0001
        -no_write                    FLAG
        -override_existence_check    CHOICES 0 1
                                     DEFAULT 0
        -override_tracking           CHOICES 0 1
                                     DEFAULT 0
    }

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rip_config $args $man_args $opt_args]
        if {[keylget returnList status] != $::SUCCESS} {
            keylset returnList log "ERROR in $procName: [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_rip_route_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rip_route_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    
    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -mode            CHOICES create modify delete
    }

    set opt_args {
        -handle
        -route_handle
        -num_prefixes       RANGE 1-1000000
        -prefix_start       IP
        -prefix_length      RANGE 1-128
        -prefix_step        IP
        -metric             RANGE 0-4294967295
        -next_hop           IP
        -route_tag          RANGE 0-65535
        -reset              FLAG
        -no_write           FLAG
    }
        
    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rip_route_config $args $man_args $opt_args]
        if {[keylget returnList status] != $::SUCCESS} {
            keylset returnList log "ERROR in $procName: [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_rip_control { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rip_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments        
    set opt_args {
        -port_handle        REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -handle
        -mode               CHOICES stop start restart flap
        -advertise
        -withdraw
    }

	if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }
    keylset returnList clicks [format "%u" $retValueClicks]
    keylset returnList seconds [format "%u" $retValueSeconds]

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rip_control $args {} $opt_args]
        keylset returnList clicks [format "%u" $retValueClicks]
        keylset returnList seconds [format "%u" $retValueSeconds]
        if {[keylget returnList status] != $::SUCCESS} {
            keylset returnList log "ERROR in $procName: [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}
